Package: slurmR
Title: A Lightweight Wrapper for 'Slurm'
Version: 0.5-4
Authors@R: c( person("George", "Vega Yon", email = "g.vegayon@gmail.com", role = c("aut",
          "cre"), comment = c(ORCID = "0000-0002-3171-0844")), person("Paul", "Marjoram",
          email = "pmarjora@usc.edu", role = c("ctb", "ths"), comment = c(ORCID =
          "0000-0003-0824-7449")), person("National Cancer Institute (NCI)", role =
          "fnd", comment = "Grant Number 5P01CA196569-02"), person("Michael", "Schubert",
          role = "rev", comment = c(what = "JOSS reviewer", ORCID =
          "0000-0002-6862-5221")), person("Michel", "Lang", role = "rev", comment =
          c(what = "JOSS reviewer", ORCID = "0000-0001-9754-0393")) )
Date: 2023-08-29
Description: 'Slurm', Simple Linux Utility for Resource Management
          <https://slurm.schedmd.com/>, is a popular 'Linux' based software used to 
          schedule jobs in 'HPC' (High Performance Computing) clusters. This R package
          provides a specialized lightweight wrapper of 'Slurm' with a syntax similar to
          that found in the 'parallel' R package. The package also includes a method for
          creating socket cluster objects spanning multiple nodes that can be used with
          the 'parallel' package.
Depends: R (>= 3.3.0), parallel
License: MIT + file LICENSE
BugReports: https://github.com/USCbiostats/slurmR/issues
URL: https://github.com/USCbiostats/slurmR, https://slurm.schedmd.com/
Encoding: UTF-8
RoxygenNote: 7.2.3
Suggests: knitr, rmarkdown, covr, tinytest
Imports: utils
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: no
Packaged: 2023-08-29 23:26:22 UTC; george
Author: George Vega Yon [aut, cre] (<https://orcid.org/0000-0002-3171-0844>),
  Paul Marjoram [ctb, ths] (<https://orcid.org/0000-0003-0824-7449>),
  National Cancer Institute (NCI) [fnd] (Grant Number 5P01CA196569-02),
  Michael Schubert [rev] (JOSS reviewer,
    <https://orcid.org/0000-0002-6862-5221>),
  Michel Lang [rev] (JOSS reviewer,
    <https://orcid.org/0000-0001-9754-0393>)
Maintainer: George Vega Yon <g.vegayon@gmail.com>
Repository: CRAN
Date/Publication: 2023-08-30 09:10:02 UTC
Built: R 4.6.0; ; 2026-04-27 03:30:29 UTC; unix
