# Generated by roxygen2: do not edit by hand

S3method("$",slurm_status)
S3method(Slurm_collect,slurm_job)
S3method(parse_flags,default)
S3method(parse_flags,list)
S3method(print,opts_slurmR)
S3method(print,slurm_cluster)
S3method(print,slurm_info)
S3method(print,slurm_job)
S3method(print,slurm_status)
S3method(sacct,default)
S3method(sacct,slurm_job)
S3method(sbatch,character)
S3method(sbatch,slurm_job)
S3method(scancel,default)
S3method(scancel,slurm_job)
S3method(squeue,default)
S3method(squeue,slurm_job)
S3method(status,default)
S3method(status,slurm_job)
S3method(stopCluster,slurm_cluster)
S3method(wait_slurm,integer)
S3method(wait_slurm,slurm_job)
export(JOB_STATE_CODES)
export(SchedulerParameters)
export(Slurm_EvalQ)
export(Slurm_Map)
export(Slurm_clean)
export(Slurm_collect)
export(Slurm_env)
export(Slurm_lapply)
export(Slurm_log)
export(Slurm_sapply)
export(WhoAmI)
export(docker_available)
export(expand_array_indexes)
export(last_job)
export(last_submitted_job)
export(makeSlurmCluster)
export(new_rscript)
export(new_slurm_job)
export(opts_slurmR)
export(parse_flags)
export(random_job_name)
export(read_sbatch)
export(read_slurm_job)
export(sacct)
export(sacct_)
export(sbatch)
export(scancel)
export(slurm.conf)
export(slurm_available)
export(slurmr_cmd)
export(slurmr_docker_pull)
export(slurmr_docker_run)
export(slurmr_docker_stop)
export(snames)
export(sourceSlurm)
export(squeue)
export(status)
export(the_plan)
export(wait_slurm)
export(whoami)
export(write_slurm_job)
importFrom(parallel,stopCluster)
importFrom(stats,setNames)
importFrom(utils,head)
importFrom(utils,sessionInfo)
importFrom(utils,str)
